@@ -1,9 +1,5 @@
-use crate::{Identity, ProfileIdentifier};
-use async_trait::async_trait;
-use ockam_core::compat::boxed::Box;
-use ockam_core::{Address, Decodable, Message, Result, Route, Routed};
-use ockam_node::Context;
-use ockam_vault_sync_core::VaultSync;
+use crate::ProfileIdentifier;
+use ockam_core::{Decodable, Message, Result, Routed};
 
 mod secure_channel_worker;
 pub(crate) use secure_channel_worker::*;
@@ -16,53 +12,6 @@ pub use trust_policy::*;
 mod local_info;
 pub use local_info::*;
 
-#[async_trait]
-pub trait SecureChannelTrait {
-    async fn create_secure_channel_async(
-        self,
-        ctx: &Context,
-        route: Route,
-        trust_policy: impl TrustPolicy,
-        vault: &Address,
-    ) -> Result<Address>;
-
-    async fn create_secure_channel_listener_async(
-        self,
-        ctx: &Context,
-        address: Address,
-        trust_policy: impl TrustPolicy,
-        vault: &Address,
-    ) -> Result<()>;
-}
-
-#[async_trait]
-impl<P: Identity + Send + Clone> SecureChannelTrait for P {
-    /// Create mutually authenticated secure channel
-    async fn create_secure_channel_async(
-        self,
-        ctx: &Context,
-        route: Route,
-        trust_policy: impl TrustPolicy,
-        vault: &Address,
-    ) -> Result<Address> {
-        let vault = VaultSync::create_with_worker(ctx, vault)?;
-        SecureChannelWorker::create_initiator(ctx, route, self, trust_policy, vault).await
-    }
-
-    /// Create mutually authenticated secure channel listener
-    async fn create_secure_channel_listener_async(
-        self,
-        ctx: &Context,
-        address: Address,
-        trust_policy: impl TrustPolicy,
-        vault: &Address,
-    ) -> Result<()> {
-        let vault = VaultSync::create_with_worker(ctx, vault)?;
-        let listener = ProfileChannelListener::new(trust_policy, self, vault);
-        ctx.start_worker(address, listener).await
-    }
-}
-
 // TODO: rename
 pub fn check_message_origin<T: Message>(
     msg: &Routed<T>,
@@ -89,8 +38,8 @@ pub fn get_secure_channel_participant_id<T: Message>(msg: &Routed<T>) -> Result<
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::{Entity, SecureChannels};
-    use ockam_core::route;
+    use crate::{Entity, Identity};
+    use ockam_core::{route, Route};
     use ockam_vault_sync_core::Vault;
 
     #[test]
@@ -98,20 +47,24 @@ mod test {
         let (mut ctx, mut executor) = ockam_node::start_node();
         executor
             .execute(async move {
-                let alice_vault = Vault::create(&ctx).expect("failed to create vault");
-                let bob_vault = Vault::create(&ctx).expect("failed to create vault");
+                let alice_vault = Vault::create(&ctx).await.expect("failed to create vault");
+                let bob_vault = Vault::create(&ctx).await.expect("failed to create vault");
 
-                let mut alice = Entity::create(&ctx, &alice_vault).unwrap();
-                let mut bob = Entity::create(&ctx, &bob_vault).unwrap();
+                let mut alice = Entity::create(&ctx, &alice_vault).await.unwrap();
+                let mut bob = Entity::create(&ctx, &bob_vault).await.unwrap();
 
-                let alice_trust_policy = TrustIdentifierPolicy::new(bob.identifier().unwrap());
-                let bob_trust_policy = TrustIdentifierPolicy::new(alice.identifier().unwrap());
+                let alice_trust_policy =
+                    TrustIdentifierPolicy::new(bob.identifier().await.unwrap());
+                let bob_trust_policy =
+                    TrustIdentifierPolicy::new(alice.identifier().await.unwrap());
 
                 bob.create_secure_channel_listener("bob_listener", bob_trust_policy)
+                    .await
                     .unwrap();
 
                 let alice_channel = alice
                     .create_secure_channel(route!["bob_listener"], alice_trust_policy)
+                    .await
                     .unwrap();
 
                 ctx.send(
@@ -123,7 +76,10 @@ mod test {
                 let msg = ctx.receive::<String>().await.unwrap().take();
 
                 let local_info = LocalInfo::decode(msg.local_message().local_info()).unwrap();
-                assert_eq!(local_info.their_profile_id(), &alice.identifier().unwrap());
+                assert_eq!(
+                    local_info.their_profile_id(),
+                    &alice.identifier().await.unwrap()
+                );
 
                 let return_route = msg.return_route();
                 assert_eq!("Hello, Bob!", msg.body());
@@ -137,7 +93,10 @@ mod test {
                 let local_info = msg.local_message().local_info();
 
                 let local_info = LocalInfo::decode(local_info).unwrap();
-                assert_eq!(local_info.their_profile_id(), &bob.identifier().unwrap());
+                assert_eq!(
+                    local_info.their_profile_id(),
+                    &bob.identifier().await.unwrap()
+                );
 
                 assert_eq!("Hello, Alice!", msg.body());
 
@@ -151,22 +110,27 @@ mod test {
         let (mut ctx, mut executor) = ockam_node::start_node();
         executor
             .execute(async move {
-                let vault = Vault::create(&ctx).unwrap();
+                let vault = Vault::create(&ctx).await.unwrap();
 
-                let mut alice = Entity::create(&ctx, &vault).unwrap();
-                let mut bob = Entity::create(&ctx, &vault).unwrap();
+                let mut alice = Entity::create(&ctx, &vault).await.unwrap();
+                let mut bob = Entity::create(&ctx, &vault).await.unwrap();
 
-                let alice_trust_policy = TrustIdentifierPolicy::new(bob.identifier().unwrap());
-                let bob_trust_policy = TrustIdentifierPolicy::new(alice.identifier().unwrap());
+                let alice_trust_policy =
+                    TrustIdentifierPolicy::new(bob.identifier().await.unwrap());
+                let bob_trust_policy =
+                    TrustIdentifierPolicy::new(alice.identifier().await.unwrap());
 
                 bob.create_secure_channel_listener("bob_listener", bob_trust_policy.clone())
+                    .await
                     .unwrap();
 
                 let alice_channel = alice
                     .create_secure_channel(route!["bob_listener"], alice_trust_policy.clone())
+                    .await
                     .unwrap();
 
                 bob.create_secure_channel_listener("bob_another_listener", bob_trust_policy)
+                    .await
                     .unwrap();
 
                 let alice_another_channel = alice
@@ -174,6 +138,7 @@ mod test {
                         route![alice_channel, "bob_another_listener"],
                         alice_trust_policy,
                     )
+                    .await
                     .unwrap();
 
                 ctx.send(
@@ -204,25 +169,30 @@ mod test {
         let (mut ctx, mut executor) = ockam_node::start_node();
         executor
             .execute(async move {
-                let vault = Vault::create(&ctx).unwrap();
+                let vault = Vault::create(&ctx).await.unwrap();
 
-                let mut alice = Entity::create(&ctx, &vault).unwrap();
-                let mut bob = Entity::create(&ctx, &vault).unwrap();
+                let mut alice = Entity::create(&ctx, &vault).await.unwrap();
+                let mut bob = Entity::create(&ctx, &vault).await.unwrap();
 
-                let alice_trust_policy = TrustIdentifierPolicy::new(bob.identifier().unwrap());
-                let bob_trust_policy = TrustIdentifierPolicy::new(alice.identifier().unwrap());
+                let alice_trust_policy =
+                    TrustIdentifierPolicy::new(bob.identifier().await.unwrap());
+                let bob_trust_policy =
+                    TrustIdentifierPolicy::new(alice.identifier().await.unwrap());
 
                 bob.create_secure_channel_listener("bob_listener", bob_trust_policy.clone())
+                    .await
                     .unwrap();
 
                 let alice_channel = alice
                     .create_secure_channel(route!["bob_listener"], alice_trust_policy.clone())
+                    .await
                     .unwrap();
 
                 bob.create_secure_channel_listener(
                     "bob_another_listener",
                     bob_trust_policy.clone(),
                 )
+                .await
                 .unwrap();
 
                 let alice_another_channel = alice
@@ -230,9 +200,11 @@ mod test {
                         route![alice_channel, "bob_another_listener"],
                         alice_trust_policy.clone(),
                     )
+                    .await
                     .unwrap();
 
                 bob.create_secure_channel_listener("bob_yet_another_listener", bob_trust_policy)
+                    .await
                     .unwrap();
 
                 let alice_yet_another_channel = alice
@@ -240,6 +212,7 @@ mod test {
                         route![alice_another_channel, "bob_yet_another_listener"],
                         alice_trust_policy,
                     )
+                    .await
                     .unwrap();
 
                 ctx.send(
@@ -270,18 +243,21 @@ mod test {
         let (mut ctx, mut executor) = ockam_node::start_node();
         executor
             .execute(async move {
-                let vault = Vault::create(&ctx).unwrap();
+                let vault = Vault::create(&ctx).await.unwrap();
 
-                let mut alice = Entity::create(&ctx, &vault).unwrap();
-                let mut bob = Entity::create(&ctx, &vault).unwrap();
+                let mut alice = Entity::create(&ctx, &vault).await.unwrap();
+                let mut bob = Entity::create(&ctx, &vault).await.unwrap();
 
-                let alice_trust_policy = TrustIdentifierPolicy::new(bob.identifier().unwrap());
-                let bob_trust_policy = TrustIdentifierPolicy::new(alice.identifier().unwrap());
+                let alice_trust_policy =
+                    TrustIdentifierPolicy::new(bob.identifier().await.unwrap());
+                let bob_trust_policy =
+                    TrustIdentifierPolicy::new(alice.identifier().await.unwrap());
 
                 let n = rand::random::<u8>() % 5 + 4;
                 let mut channels = vec![];
                 for i in 0..n {
                     bob.create_secure_channel_listener(i.to_string(), bob_trust_policy.clone())
+                        .await
                         .unwrap();
                     let channel_route: Route;
                     if i > 0 {
@@ -291,6 +267,7 @@ mod test {
                     }
                     let alice_channel = alice
                         .create_secure_channel(channel_route, alice_trust_policy.clone())
+                        .await
                         .unwrap();
                     channels.push(alice_channel);
                 }
